@@ -1,7 +1,12 @@
 # -*- coding: utf-8 -*-
+try:
+    import cPickle as pickle
+except:
+    import pickle
 import json
 import urllib
 import weakref
+import warnings
 from lucenequerybuilder import Q
 
 import options
@@ -283,7 +288,7 @@ class Transaction(object):
             for operation in self.operations:
                 on_object = operation.get_object()
                 if hasattr(on_object, "update"):
-                    on_object.update(extensions=False)
+                    on_object.update(extensions=False, delete_on_not_found=True)
         # Objects to return
         for referenced_object in self.references:
                 ref_object = referenced_object()
@@ -379,7 +384,7 @@ class Base(object):
     def _safe_string(self, s):
         return unicode(s.decode("utf-8"))
 
-    def update(self, extensions=True):
+    def update(self, extensions=True, delete_on_not_found=False):
         response, content = Request().get(self.url)
         if response.status == 200:
             self._dic.update(json.loads(content).copy())
@@ -387,6 +392,11 @@ class Base(object):
                 self._extensions = self._dic.get('extensions', {})
                 if self._extensions:
                     self.extensions = ExtensionsProxy(self._extensions)
+        elif delete_on_not_found and response.status == 404:
+            self.url = None
+            self._dic = {}
+            self = None
+            del self
         else:
             raise NotFoundError(response.status, "Unable get object")
 
@@ -407,8 +417,7 @@ class Base(object):
                                                    "relationships?)")
 
     def __getitem__(self, key, tx=None):
-        property_url = self._dic["property"].replace("{key}",
-                                                     urllib.quote_plus(key))
+        property_url = self._dic["property"].replace("{key}", smart_quote(key))
         tx = Transaction.get_transaction(tx)
         if tx:
             return tx.subscribe(TX_GET, property_url, obj=self)
@@ -447,8 +456,7 @@ class Base(object):
         if isinstance(value, Transaction):
             tx = tx or value
             value = value.get_value()
-        property_url = self._dic["property"].replace("{key}",
-                                                     urllib.quote_plus(key))
+        property_url = self._dic["property"].replace("{key}", smart_quote(key))
         tx = Transaction.get_transaction(tx)
         if tx:
             transaction_url = self._dic["property"].replace("{key}", "")
@@ -468,7 +476,7 @@ class Base(object):
         self.__setitem__(key, value)
 
     def __delitem__(self, key, tx=None):
-        property_url = self._dic["property"].replace("{key}", key)
+        property_url = self._dic["property"].replace("{key}", smart_quote(key))
         tx = Transaction.get_transaction(tx)
         if tx:
             return tx.subscribe(TX_DELETE, property_url, obj=self)
@@ -491,10 +499,13 @@ class Base(object):
         return self._dic["data"].__iter__()
 
     def __eq__(self, obj):
-        return (hasattr(obj, "url")
-                and self.url == obj.url
-                and hasattr(obj, "__class__")
-                and self.__class__ == obj.__class__)
+        if not self.url and not self._dic:
+            return (obj == None)
+        else:
+            return (hasattr(obj, "url")
+                    and self.url == obj.url
+                    and hasattr(obj, "__class__")
+                    and self.__class__ == obj.__class__)
 
     def __ne__(self, obj):
         return not self.__cmp__(obj)
@@ -509,7 +520,10 @@ class Base(object):
         return self.__unicode__()
 
     def __unicode__(self):
-        return u"<Neo4j %s: %s>" % (self.__class__.__name__, self.url)
+        if not self.url and not self._dic:
+            return None
+        else:
+            return u"<Neo4j %s: %s>" % (self.__class__.__name__, self.url)
 
     def _get_properties(self):
         return self._dic["data"]
@@ -671,6 +685,10 @@ class Node(Base):
         """
         HACK: Allow to set node relationship
         """
+        warnings.warn("Deprecated, in favor of pythonic style to declare "
+                      "relationships: n2.relationships.create(rel_name, n2). "
+                      "This is needed in order to handle pickling in nodes.",
+                      DeprecationWarning)
 
         def relationship(to, *args, **kwargs):
             tx = Transaction.get_transaction(kwargs.get("tx", None))
@@ -699,6 +717,21 @@ class Node(Base):
                 raise StatusException(response.status, "Invalid data sent")
         return relationship
 
+    # HACK: Special methods for handle pickling manually
+    def __getstate__(self):
+        data = {}
+        for key, value in self.__dict__.items():
+            try:
+                encoded = pickle.dumps(value)
+            except pickle.PicklingError:
+                encoded = pickle.dumps(pickle.Unpickable())
+            data[key] = encoded
+        return data
+
+    def __setstate__(self, state):
+        for key, value in state.items():
+            self.__dict__[key] = pickle.loads(value)
+
     def _get_relationships(self):
         """
         HACK: Return a 3-methods class: incoming, outgoing and all.
@@ -953,7 +986,7 @@ class Index(object):
             self.url = url
 
         def __getitem__(self, value):
-            url = "%s/%s" % (self.url, urllib.quote_plus(value))
+            url = "%s/%s" % (self.url, smart_quote(value))
             return Index._get_results(url, self._index_for)
 
         def __setitem__(self, value, item):
@@ -966,7 +999,7 @@ class Index(object):
                 raise TypeError("%s is a %s and the index is for %ss"
                                 % (item, self._index_for.capitalize(),
                                    self._index_for))
-            value = urllib.quote_plus(value)
+            value = smart_quote(value)
             if isinstance(item, Base):
                 url_ref = item.url
             else:
@@ -986,7 +1019,7 @@ class Index(object):
                                       ", data %s" % (request_url, url_ref))
 
         def query(self, value):
-            url = "%s?query=%s" % (self.url, urllib.quote_plus(value))
+            url = "%s?query=%s" % (self.url, smart_quote(value))
             return Index._get_results(url, self._index_for)
 
     def __init__(self, index_for, name, **kwargs):
@@ -1016,9 +1049,9 @@ class Index(object):
         self.get(key)[value] = item
 
     def get(self, key, value=None):
-        key = urllib.quote(key)
+        key = smart_quote(key)
         if value:
-            value = urllib.quote_plus(value)
+            value = smart_quote(value)
             return self.IndexKey(self._index_for,
                                  "%s/%s" % (self.url, key))[value]
         else:
@@ -1031,13 +1064,13 @@ class Index(object):
             if not isinstance(item, Base):
                 raise TypeError("%s has no url attribute" % item)
             if key and value:
-                key = urllib.quote(key)
-                value = urllib.quote_plus(value)
+                key = smart_quote(key)
+                value = smart_quote(value)
                 url = self.template.replace("{key}", key).replace("{value}",
                                                                   value)
                 url = "%s/%s" % (url, item.id)
             elif key and not value:
-                key = urllib.quote(key)
+                key = smart_quote(key)
                 url = "%s/%s" % (self.template.replace("{key}", key), item.id)
             elif not key and not value:
                 url = self.template.replace("{key}/{value}", item.id)
@@ -1195,7 +1228,12 @@ class Relationships(object):
         return self.__getattr__("all")(tx=tx)[index]
 
     def create(self, relationship_name, to, **kwargs):
-        return getattr(self._node, relationship_name)(to, **kwargs)
+        # TODO: Improve the unicode checking
+        try:
+            return getattr(self._node, relationship_name)(to, **kwargs)
+        except (KeyError, UnicodeEncodeError, UnicodeError):
+            safe_name = smart_quote(relationship_name)
+            return getattr(self._node, safe_name)(to, **kwargs)
 
     def get(self, index, tx=None):
         tx = Transaction.get_transaction(tx)
@@ -1279,6 +1317,9 @@ class BaseInAndOut(object):
 
     def __init__(self, direction):
         self.direction = direction
+        if direction == "both":
+            warnings.warn("Deprecated, use \"All\" ('both') instead.",
+                          DeprecationWarning)
 
     def get(self, attr):
         return self.__getattr__(attr)
@@ -1461,3 +1502,12 @@ class Extension(object):
             else:
                 params_kwargs[param] = value
         return params_kwargs
+
+
+def smart_quote(val):
+    # TODO: Improve the unicode checking
+    try:
+        safe_key = urllib.quote(val, safe="")
+    except (KeyError, UnicodeEncodeError, UnicodeError):
+        safe_key = urllib.quote(val.encode("utf8"), safe="")
+    return safe_key
